//-------------------------------------------------------------------
// WDTimerFuncs.cpp
//
// Functions to enable and reset the StrongARM watchdog timer.
//
// StrongARM timer register oscr counts up at 3.6864 MHz.  This
// driver enables timer match register 3 (osmr3) so that when
// the system timer matches the value set, the system performs
// a hardware reset.
//
// Once the timer has been armed, the calling application must
// reset the timer at least as often as the period set when
// the timer was initialized.  Otherwise the system will reset.
//
// This library intentionally does not include a function to disable 
// the watchdog timer.  Once this function has been called, the 
// watchdog timer can only disabled by resetting the processor.
//
//
// (c)2002 Applied Data Systems
//
// 4dec02 cl
//  - initial release
// 17dec02 ak
//  - add headers and comments
//  - WDTimerReset takes no parameters; just resets to original value
//-------------------------------------------------------------------

#include "stdafx.h"
#include "SA1110.hpp"

SA1100_OST *OstRegPtr=NULL;
static ULONG WatchdogTimerGuardValue = 0;

//-------------------------------------------------------------------
// Initializes the watchdog timer to generate a system reset
// in TimerValue seconds.  Call the WDTimerReset
//
// For reliable operation, use a TimerValue of less than 1165--the
// roll-over period for the oscr counter.  Typical watchdog settings 
// are 5 to 30 seconds.
//
//-------------------------------------------------------------------
BOOL WDTimerEnable(ULONG TimerValue)
{
	LONG oscr;
	LONG osmr3;
	ULONG oier;

	WatchdogTimerGuardValue =(LONG)(TimerValue*INPUT_CLOCK);

	// Read Current OS Timer Counter
	oscr = (LONG)READ_REGISTER_ULONG(&OstRegPtr->OSCR);
	DEBUGMSG(1,(L"WDTimerEnable:oscr=0x%x \r\n",oscr));

	osmr3 = oscr + WatchdogTimerGuardValue;
	WRITE_REGISTER_ULONG(&OstRegPtr->OSMR3,osmr3);
	DEBUGMSG(1,(L"WDTimerEnable:osmr3=0x%x \r\n",osmr3));

	// Clear Dummy Status
	//--------------------
	WRITE_REGISTER_ULONG(&OstRegPtr->OSSR,OSSR_M3);

	// Enable OSTimer 3
	//------------------
	oier = READ_REGISTER_ULONG(&OstRegPtr->OIER);
    oier |= OIER_E3;
    WRITE_REGISTER_ULONG(&OstRegPtr->OIER,oier);

	// Enable Watchdog Timer
	//-----------------------
	WRITE_REGISTER_ULONG(&OstRegPtr->OWER,OWER_WME);	
	return TRUE;
}

//-------------------------------------------------------------------
// Re-arms the watchdog timer to reset the system in
// WatchdogTimerGuardValue ticks from now.
//-------------------------------------------------------------------
//BOOL WDTimerUpdate(ULONG TimerValue)	// sec
BOOL WDTimerReset (void)
{
	LONG oscr;	
	LONG osmr3;

	// Read Current OS Timer Counter
	oscr = (LONG)READ_REGISTER_ULONG(&OstRegPtr->OSCR);

  // New timeout is WatchdogTimerGuardValue ticks from now
//	osmr3 =  oscr + (LONG)(TimerValue*INPUT_CLOCK) + WatchdogTimerGuardValue;
	osmr3 =  oscr + WatchdogTimerGuardValue;

	// Update OSTimer Match	Register
	WRITE_REGISTER_ULONG(&OstRegPtr->OSMR3,osmr3);

	return TRUE;
}
